package metrics;

import java.io.File;
import java.util.ArrayList;
import java.util.List;

/**
 * Holds the data belonging to a project
 */
public class Project {
	private final String name;
	private final String sourcePath;
	private transient File sourceDir;
	private String libraryPath;
	private transient File libraryDir;
	private List<String> libraries = new ArrayList<String>();
	
	public Project(String name, String sourcePath) {
		this.name = name;
		this.sourcePath = sourcePath;
	}
	
	public Project(String name, String sourcePath, String libraryPath, String[] libraries) {
		this(name, sourcePath);
		this.libraryPath = libraryPath;
		if(libraries != null) {
			for(String library : libraries) {
				this.libraries.add(libraryPath+File.separator+library);
			}
		}
	}

	public String getName() {
		return name;
	}

	public String getSourcePath() {
		return sourcePath;
	}
	
	public File getSourceDirectory() {
		if(sourceDir == null) {
			sourceDir = new File(sourcePath);
		}
		return sourceDir;
	}
	
	public boolean hasLibraryPath() {
		return libraryPath != null;
	}

	public String getLibraryPath() {
		return libraryPath;
	}
	
	public File getLibraryDirectory() {
		if(libraryDir == null) {
			libraryDir = new File(libraryPath);
		}
		return libraryDir;
	}

	public List<String> getLibraries() {
		return libraries;
	}
	
}
