package metrics.hyperonymy.potential;

import java.io.PrintWriter;

import AST.ASTNode;
import metrics.Metric;

public class Variable implements Metric<AST.Variable> {

	public String getName() {
		return "hyp-pot-var";
	}

	public void setupHeader(PrintWriter writer) {
		writer.println("project;location;varname;kind of variable;synthetic;type");
	}
	
	public void calculate(AST.Variable node) {
		if(node.name().startsWith("the")) {
			System.err.println(node.hostType().fullName()+"."+node.name()+":"+node.type().name());
		}
		String kindOfVariable = Variable.getKindOfVariable(node);
		((ASTNode<?>)node).logMetric(this, node.name(), kindOfVariable, node.isSynthetic(),
				node.type().fullName());
	}

	public static String getKindOfType(AST.TypeDecl node) {
		if(node.isArrayDecl()) {
			return "array";
		} else if(node.isClassDecl()) {
			return "class";
		} else if(node.isInterfaceDecl()) {
			return "interface";
		} else if(node.isNull()) {
			return "null";
		} else if(node.isPrimitiveType()) {
			return "primitive";
		} else if(node.isVoid()) {
			return "void";
		} else if(node.isTypeVariable()) {
			return "type variable";
		} else {
			throw new RuntimeException("Unknown kind of type: "+node.getClass().getName());
		}
	}
	
	public static String getKindOfVariable(AST.Variable node) {
		String type = null;
		if(node.isClassVariable()) {
			type = "class variable";
		} else if(node.isInstanceVariable()) {
			type = "instance variable";
		} else if(node.isLocalVariable()) {
			type = "local variable";
		} else if(node.isConstructorParameter()) {
			type = "constructor parameter";
		} else if(node.isMethodParameter()) {
			type = "method parameter";
		} else if(node.isExceptionHandlerParameter()) {
			type = "exception handler parameter";
		} else {
			throw new RuntimeException("Unknown kind of variable: "+node.getClass().getName());
		}
		return type;
	}

}
