package metrics.meronymy.potential;

import java.io.PrintWriter;

import metrics.Metric;
import AST.ASTNode;
import AST.Block;
import AST.BodyDecl;
import AST.ConstructorDecl;
import AST.InstanceInitializer;
import AST.MethodDecl;
import AST.StaticInitializer;

public class Stmt implements Metric<BodyDecl> {

	public String getName() {
		return "mer-pot-stmt";
	}

	public void setupHeader(PrintWriter writer) {
		writer.println("project;location;kind of declaration;number of statements");
	}
	
	public void calculate(BodyDecl node) {
		if(node instanceof ConstructorDecl) {
			if(((ConstructorDecl)node).getBlock().getNumStmt() > 0) {
				calculate(((ConstructorDecl)node).getBlock(), "constructor");
			}
		} else if(node instanceof InstanceInitializer) {
			if(((InstanceInitializer)node).getBlock().getNumStmt() > 0) {
				calculate(((InstanceInitializer)node).getBlock(), "instance initializer");
			}
		} else if(node instanceof MethodDecl) {
			if(((MethodDecl)node).hasBlock() && ((MethodDecl)node).getBlock().getNumStmt() > 0) {
				calculate(((MethodDecl)node).getBlock(), "method");
			}
		} else if(node instanceof StaticInitializer) {
			if(((StaticInitializer)node).getBlock().getNumStmt() > 0) {
				calculate(((StaticInitializer)node).getBlock(), "static initializer");
			}
		} // else: FieldDecl, FieldDeclaration, MemberTypeDecl do not have statements
		  // or should not be handled at this level because they will handled when
		  // collectMetrics() recurses down to the members directly
	}
	
	private void calculate(Block block, String kindOfDeclaration) {
		java.util.List<ASTNode<?>> stmts = block.findChildrenRecursively(AST.Stmt.class);
		block.logMetric(this, kindOfDeclaration, stmts.size());
	}

}
