package metrics.meronymy.potential;

import java.io.PrintWriter;

import metrics.Metric;
import AST.BodyDecl;
import AST.ConstructorDecl;
import AST.FieldDecl;
import AST.FieldDeclaration;
import AST.InstanceInitializer;
import AST.MethodDecl;
import AST.StaticInitializer;

public class TLStmt implements Metric<BodyDecl> {

	public String getName() {
		return "mer-pot-numTLStmt";
	}

	public void setupHeader(PrintWriter writer) {
		writer.println("project;location;number of top-level statements");
	}
	
	public void calculate(BodyDecl node) {
		if(node instanceof ConstructorDecl) {
			calculate((ConstructorDecl)node);
		} else if(node instanceof InstanceInitializer) {
			calculate((InstanceInitializer)node);
		} else if(node instanceof MethodDecl) {
			calculate((MethodDecl)node);
		} else if(node instanceof StaticInitializer) {
			calculate((StaticInitializer)node);
		} // else: FieldDecl, FieldDeclaration, MemberTypeDecl do not have statements
		  // or should not be handled at this level because they will handled when
		  // collectMetrics() recurses down to the members directly
	}
	
	public void calculate(ConstructorDecl node) {
		if(node.getBlock().getNumStmt() > 0) {
			node.logMetric(this, node.getBlock().getNumStmt());
		}
	}
	
	public void calculate(InstanceInitializer node) {
		if(node.getBlock().getNumStmt() > 0) {
			node.logMetric(this, node.getBlock().getNumStmt());
		}
	}
	
	public void calculate(MethodDecl node) {
		if(node.hasBlock() && node.getBlock().getNumStmt() > 0) {
			node.logMetric(this, node.getBlock().getNumStmt());
		}
	}
	
	public void calculate(StaticInitializer node) {
		if(node.getBlock().getNumStmt() > 0) {
			node.logMetric(this, node.getBlock().getNumStmt());
		}
	}

}
