package tests;

import static org.junit.Assert.*;

import java.io.File;

import org.junit.Test;

public class Kind2 extends TestsContainer {

	// utility members
	
	@Override
	protected String getTestDir() {
		return "tests"+File.separator+getTestPackage()+File.separator;
	}

	@Override
	protected String getTestPackage() {
		return "kind2";
	}
	
	// tests
	
	@Test
	public void test01ReadNonExistentField() throws Exception {
		runNegativeTest(01, "It should not be possible to anchor in a non-existent field",
				"Errors:\n"
				+ "tests/kind2/test01/Test01.jaaa:6:\n"
				+ "  Semantic Error: Cannot anchor indirect anaphor .Integer: no suitable anchor found\n");
	}
	
	@Test
	public void test02ReadPublicFieldFromVariableDeclaration() throws Exception {
		runPositiveTestAndAssertEquals(2, "Should have referred to public field",
				"Wrong IA value", 2);
	}
	
	// TODO: Inherited field
    // TODO: - non-static fields in static context
	// TODO: static fields in static/non-static context
	// TODO: Referential ambiguity due to multiple matching fields
	
	/*
	@Test
	public void test06ReadPrivateField() throws Exception {
		fail("Not yet implemented");
		// runNegativeTest
	}
	*/
	
	// Test anchoring:
	// TODO: VarDeclStmt (1) -> (n) VarDeclaration,
	//       FieldDeclaration (1) -> (n) VarDeclaration
	//       VarAccess
	//       VariableDeclaration (with and without initial value, simple and qualified initial values)
	//       ParameterDeclaration (at least in catch clause etc.)
	// 		 MethodAccess
	//		 ClassInstanceExpr
	
	/*
	@Test
	public void test07ReadFieldThatHasASubtypeOfTheAnaphor() throws Exception {
		fail("Not yet implemented");
	} */
	
	// TODO: Need to cast the result of the IA? (due to static method resolution)
	
	// TODO: setting the value of a field
	
	// TODO: isVariable()
	
	@Test
	public void test10ReferentialAmbiguity() throws Exception {
		runNegativeTest(10, "Referential ambiguity expected",
				"Errors:\n"
				+ "tests/kind2/test10/Test10.jaaa:9:\n"
				+ "  Semantic Error: Cannot anchor indirect anaphor .Integer: more than 1 suitable anchor:\n"
				+ "  line 7:7: Test10 a = new Test10(); in {\n"
				+ "    Test10 a = new Test10();\n"
				+ "    Test10 b = new Test10();\n"
				+ "    return .Integer;\n"
				+ "  }\n"
				+ "  line 8:7: Test10 b = new Test10(); in {\n"
				+ "    Test10 a = new Test10();\n"
				+ "    Test10 b = new Test10();\n"
				+ "    return .Integer;\n"
				+ "  }\n");
	}
	
	// TODO: This test is ineffective right now because ClassInstanceExpr is not currently
	// matched by type2 IAs ... need to use a field access or parameter access to initialize
	 
	@Test
	public void test11PreventedReferentialAmbiguityWhenAnchoringInLocalVariableDeclarationWithInitializer() throws Exception {
		runPositiveTestAndAssertEquals(11, "Referential unambiguity could not be prevented",
				"Wrong IA value", 11);
	}
	
	// TODO: LATER: Expand tests for forms of parameters, VarDeclaration, ...

}
