package tests.metrics;

import static org.junit.Assert.*;

import java.io.File;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

import metrics.Project;

import org.junit.Test;

import tests.CompileHelper;
import AST.ASTNode;
import AST.Program;

public class Metrics {
	
	private static final String METRICS_PATH = "/Users/Basti/Documents/studium/Fernuni Hagen/WS10_11/29000/tex/metricsOutput/";
	private static final String INPUT_DIR = "/Users/Basti/Documents/studium/Fernuni Hagen/WS10_11/29000/tex/metricsInput/";
	private static final List<Project> Projects = new ArrayList<Project>(); 
	
	static {
		// initialize projects
		Projects.add(new Project(
				"junit",
				INPUT_DIR+"junit/junit-4.8.2-src/"));
		Projects.add(new Project(
				"jhotdraw",
				INPUT_DIR+"jhotdraw/JHotDraw 7.0.8/src/main/java/",
				INPUT_DIR+"jhotdraw/JHotDraw 7.0.8/lib/",
				new String[] {
						"java_30.zip",
						"libquaqua.jnilib",
						"MRJAdapter.jar",
						"quaqua.jar"
				}));
	}
	
	@Test
	public void collectMetrics() {
		ASTNode.installMetrics();
		for(Project project : Projects) {
			System.err.println("calculating metrics for "+project.getName());
			// ensure project accessibility
			assertTrue(project.getSourceDirectory().exists());
			
			// create AST
			Program program = null;
			if(project.hasLibraryPath()) {
				program = CompileHelper.compileAllJavaFilesUnder(project.getSourceDirectory(),
					project.getLibraryDirectory(), project.getLibraries());
			} else {
				program = CompileHelper.compileAllJavaFilesUnder(project.getSourceDirectory());
			}
			assertNotNull(program);
			
			// collect metrics
			program.setMetricsPath(METRICS_PATH);
			program.setMetricsProjectName(project.getName());
			program.collectMetrics();
		}
		ASTNode.finishMetrics();
	}

}
