package tests.metrics.hyperonymy.potential;

import static org.junit.Assert.*;
import static org.junit.Assert.assertEquals;
import static org.junit.Assert.assertNotNull;

import java.io.BufferedReader;
import java.io.IOException;
import java.io.PrintWriter;
import java.io.StringReader;
import java.io.StringWriter;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.List;
import java.util.Vector;

import org.junit.Test;

import tests.CompileHelper;
import tests.metrics.MetricsTest;
import AST.ASTNode;
import AST.Program;

public class AccessibilityTest implements MetricsTest {
	
	private static final String SAMPLE_1 = SAMPLES_PATH+"Java02.java";
	
	@Test
	public void test() throws IOException {
		// Setup
		StringWriter accWriterBack = new StringWriter();
		ASTNode.setPrintWriter("hyp-pot-acc", new PrintWriter(accWriterBack));
		ASTNode.installMetrics();
		
		// Run
		Program program = CompileHelper.compile(SAMPLE_1);
		assertNotNull(program);
		program.setMetricsPath("");
		program.setMetricsProjectName("projectName");
		program.collectMetrics();
		ASTNode.finishMetrics();
		
		// Examine
		List<String> expectedAcc = new Vector<String>(Arrays.asList(new String[] {
			// Java02a.Java02a()
			"projectName;"+SAMPLE_1+":0;constructor;strings;class variable;true;false;false;interface;java.util.List<java.lang.String>;[java.lang.Iterable, java.util.Collection, java.util.List];3",
			"projectName;"+SAMPLE_1+":0;constructor;field;instance variable;false;false;false;class;java.lang.String;[java.io.Serializable, java.lang.CharSequence, java.lang.Comparable, java.lang.String];4",
			
			// Java02.<clinit>()
			"projectName;"+SAMPLE_1+":19;static initializer;strings;class variable;true;true;false;interface;java.util.List<java.lang.String>;[java.lang.Iterable, java.util.Collection, java.util.List];3",
			"projectName;"+SAMPLE_1+":19;static initializer;serialVersionUID;class variable;true;false;false;primitive;@primitive.long;[@primitive.long];1",
			"projectName;"+SAMPLE_1+":19;static initializer;hello;instance variable;false;false;false;class;java.lang.String;[java.lang.CharSequence, java.lang.String];2",
			"projectName;"+SAMPLE_1+":19;static initializer;siLocal;local variable;false;false;false;class;java.util.Date;[java.lang.Cloneable, java.util.Date];2",
			
			// Java02.Java02(Integer,int,Float[],float[])
			"projectName;"+SAMPLE_1+":24;constructor;strings;class variable;true;true;false;interface;java.util.List<java.lang.String>;[java.lang.Iterable, java.util.Collection, java.util.List];3",
			"projectName;"+SAMPLE_1+":24;constructor;serialVersionUID;class variable;true;false;false;primitive;@primitive.long;[@primitive.long];1",
			"projectName;"+SAMPLE_1+":24;constructor;hello;instance variable;false;false;false;class;java.lang.String;[java.lang.CharSequence, java.lang.String];2",
			"projectName;"+SAMPLE_1+":24;constructor;count;constructor parameter;false;false;false;class;java.lang.Integer;[java.lang.Integer, java.lang.Number];2",
			"projectName;"+SAMPLE_1+":24;constructor;primitiveCount;constructor parameter;false;false;false;primitive;@primitive.int;[@primitive.int];1",
			"projectName;"+SAMPLE_1+":24;constructor;weights;constructor parameter;false;false;false;array;Float[];[Float[]];1",
			"projectName;"+SAMPLE_1+":24;constructor;primitiveWeights;constructor parameter;false;false;false;array;float[];[float[]];1",
			"projectName;"+SAMPLE_1+":24;constructor;t;exception handler parameter;false;false;false;class;java.lang.Throwable;[java.lang.Throwable];1",
			
			// Java02.hello(Object)
			"projectName;"+SAMPLE_1+":32;method;strings;class variable;true;true;false;interface;java.util.List<java.lang.String>;[java.lang.Iterable, java.util.Collection, java.util.List];3",
			"projectName;"+SAMPLE_1+":32;method;serialVersionUID;class variable;true;false;false;primitive;@primitive.long;[@primitive.long];1",
			"projectName;"+SAMPLE_1+":32;method;hello;instance variable;false;false;false;class;java.lang.String;[java.io.Serializable, java.lang.CharSequence, java.lang.Comparable, java.lang.String];4",
			"projectName;"+SAMPLE_1+":32;method;stars;method parameter;false;false;false;class;java.lang.Object;[];0",
			
			// Java02b.Java02b()
			"projectName;"+SAMPLE_1+":40;constructor;strings;class variable;true;true;false;interface;java.util.List<java.lang.String>;[java.lang.Iterable, java.util.Collection, java.util.List];3",
			"projectName;"+SAMPLE_1+":40;constructor;serialVersionUID;class variable;true;true;false;primitive;@primitive.long;[@primitive.long];1",
			"projectName;"+SAMPLE_1+":40;constructor;myWay;instance variable;false;false;false;class;java.lang.String;[java.lang.CharSequence, java.lang.Comparable, java.lang.String];3",

			// Java02c.Java02c()
			"projectName;"+SAMPLE_1+":45;constructor;strings;class variable;true;true;false;interface;java.util.List<java.lang.String>;[java.lang.Iterable, java.util.Collection, java.util.List];3",
			"projectName;"+SAMPLE_1+":45;constructor;serialVersionUID;class variable;true;true;false;primitive;@primitive.long;[@primitive.long];1",
			"projectName;"+SAMPLE_1+":45;constructor;hello;instance variable;false;true;false;class;java.lang.String;[java.lang.CharSequence, java.lang.String];2"
		}));
		BufferedReader logReader = new BufferedReader(new StringReader(accWriterBack.getBuffer().toString()));
		for(int i=1;expectedAcc.size() > 0;i++) {
			String line = logReader.readLine();
			if(line == null) {
				break;
			} else {
				if(expectedAcc.contains(line)) {
					expectedAcc.remove(line);
				} else {
					fail("the "+i+". log line:\n"+line+"\nwas not expected anymore:\n"+listInLogFormat(expectedAcc));
				}
			}
		}
		
		if(!expectedAcc.isEmpty()) {
			fail(expectedAcc.size()+" log lines were expected but did not occur:\n"+listInLogFormat(expectedAcc));
		}
	}
	
	private static String listInLogFormat(List<String> list) {
		StringBuffer buffer = new StringBuffer();
		for(int i=0;i<list.size();i++) {
			buffer.append(list.get(i)+"\n");
		}
		return buffer.toString();
	}
}
